@@ -413,11 +413,11 @@ void MsnNotificationConnection::gotAdd(const QStringList& command)
   if(contact == 0)
   {
     // Add the contact if he/she does not exist.
-    if     (listType = "FL") listNumber = Contact::MSN_LIST_FRIEND;
-    else if(listType = "AL") listNumber = Contact::MSN_LIST_ALLOWED;
-    else if(listType = "BL") listNumber = Contact::MSN_LIST_BLOCKED;
-    else if(listType = "RL") listNumber = Contact::MSN_LIST_REVERSE;
-    else                     listNumber = 0;
+    if     (listType == "FL") listNumber = Contact::MSN_LIST_FRIEND;
+    else if(listType == "AL") listNumber = Contact::MSN_LIST_ALLOWED;
+    else if(listType == "BL") listNumber = Contact::MSN_LIST_BLOCKED;
+    else if(listType == "RL") listNumber = Contact::MSN_LIST_REVERSE;
+    else                      listNumber = 0;
 
     contact = contactList_->addContact(handle, friendlyName, listNumber, groupId);
   }
