@@ -30,6 +30,7 @@
 #include "../contact/contact.h"
 #include "../contact/contactlist.h"
 #include "../currentaccount.h"
+#include "../kmessapplication.h"
 #include "../kmessdebug.h"
 #include "../msnobject.h"
 #include "msnnotificationconnection.h"
@@ -289,20 +290,23 @@ void MsnSwitchboardConnection::contactJoined(QString& handle, QString& friendlyN
 
   // Download the display picture of the contact if he/she has an msnobject
   // and the capabilities field indicates it supports MSNC1
-        Contact   *contact   = chatInfo_->getContactList()->getContactByHandle(handle);
-  const MsnObject *msnObject = contact->getMsnObject();
+  Contact   *contact   = chatInfo_->getContactList()->getContactByHandle(handle);
 
-  // If the contact is capable of transferring pictures, 
-  if(contact->getCapabilities() >= Contact::MSN_CAP_MSN60)
+  if(contact != 0)
   {
-    // Handle updates
-    connect(contact, SIGNAL(        changedMsnObject(Contact*) ) ,
-            this,      SLOT( contactChangedMsnObject(Contact*) ) );
-
-    // Download the current picture the contact has one
-    if(msnObject != 0)
+    // If the contact is capable of transferring pictures,
+    if(contact->getCapabilities() >= Contact::MSN_CAP_MSN60)
     {
-      startPictureDownload( contact );
+      // Handle updates
+      connect(contact, SIGNAL(        changedMsnObject(Contact*) ) ,
+              this,      SLOT( contactChangedMsnObject(Contact*) ) );
+
+      // Download the current picture the contact has one
+      const MsnObject *msnObject = contact->getMsnObject();
+      if(msnObject != 0)
+      {
+        startPictureDownload( contact );
+      }
     }
   }
 
@@ -316,7 +320,6 @@ void MsnSwitchboardConnection::contactJoined(QString& handle, QString& friendlyN
 void MsnSwitchboardConnection::contactLeft(const QString& handle)
 {
   QValueList<QString>::iterator it;
-  Contact *contact = chatInfo_->getContactList()->getContactByHandle(handle);
 
   // Check if the contact is in the chat..
   if ( isContactInChat( handle ) )
@@ -344,6 +347,7 @@ void MsnSwitchboardConnection::contactLeft(const QString& handle)
 
 
   // Unregister the contact
+  Contact *contact = chatInfo_->getContactList()->getContactByHandle(handle);
   if(contact != 0)
   {
     // Disconnect the update signal
@@ -609,6 +613,17 @@ void MsnSwitchboardConnection::detachContacts()
 {
   if(contactsInChat_.size() == 0) return;
 
+  // KMessApplication also defines sessionSaving() for KDE 3.1
+  KMessApplication *app = static_cast<KMessApplication *>(kapp);
+
+  // Don't call contact->setMsnObjectChat,
+  // contact may be deleted already
+  if(app->quitSelected() ||
+     app->sessionSaving())
+  {
+    return;
+  }
+
 #ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
   kdDebug() << "Switchboard::detachContacts: releasing all contacts." << endl;
 #endif
@@ -628,7 +643,7 @@ void MsnSwitchboardConnection::detachContacts()
     contact = chatInfo_->getContactList()->getContactByHandle( *it );
 
     // Are we the switchboard that transferred the contact picture?
-    if(contact->getMsnObjectChat() == this->chatInfo_)
+    if(contact != 0 && contact->getMsnObjectChat() == this->chatInfo_)
     {
       // Unregister this chat, allow others to take ownership
       contact->setMsnObjectChat(0);
@@ -653,6 +668,14 @@ void MsnSwitchboardConnection::fileTransferred(QString filename)
 // Forward an application message by reemitting it as a signal.
 void MsnSwitchboardConnection::forwardAppInitMessage(QString message)
 {
+  if(autoDeleteLater_)
+  {
+#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
+    kdDebug() << "MsnSwitchboardConnection::forwardAppInitMessage: ignoring application message while closing connection"
+              << " (message=" << message << ")." << endl;
+#endif
+    return;
+  }
   emit showAppMessage( "<font color=blue>" + message + "</font>" );
 }
 
@@ -661,6 +684,14 @@ void MsnSwitchboardConnection::forwardAppInitMessage(QString message)
 // Forward an application message by reemitting it as a signal.
 void MsnSwitchboardConnection::forwardAppMessage(QString message)
 {
+  if(autoDeleteLater_)
+  {
+#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
+    kdDebug() << "MsnSwitchboardConnection::forwardAppMessage: ignoring application message while closing connection"
+              << " (message=" << message << ")." << endl;
+#endif
+    return;
+  }
   emit showAppMessage( "<font color=purple>" + message + "</font>" );
 }
 
@@ -669,6 +700,14 @@ void MsnSwitchboardConnection::forwardAppMessage(QString message)
 // Forward an system message
 void MsnSwitchboardConnection::forwardSystemMessage(QString message)
 {
+  if(autoDeleteLater_)
+  {
+#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
+    kdDebug() << "MsnSwitchboardConnection::forwardSystemMessage: ignoring application message while closing connection"
+              << " (message=" << message << ")." << endl;
+#endif
+    return;
+  }
   emit systemMessage(message);
 }
 
@@ -1693,6 +1732,8 @@ void MsnSwitchboardConnection::sendChatMessage(QString text)
 // Send a message to the contact(s), or leave it pending until a connection is restored
 void MsnSwitchboardConnection::sendMimeMessageWhenReady(AckType ackType, const MimeMessage &message)
 {
+  if(chatInfo_ == 0) return;
+
   // If the connection is ready, send the message
   if( isConnected() && contactsConnected_ )
   {
@@ -1926,7 +1967,7 @@ void MsnSwitchboardConnection::startFileTransfer(const QString &handle)
 
     // FIXME: Just get the first contact, we'll support more later. (see ChatWindow::transferFile() for details)
     Contact *contact = chatInfo_->getContactList()->getContactByHandle(handle);
-    uint clientCaps = contact->getCapabilities();
+    uint clientCaps = (contact != 0 ? contact->getCapabilities() : 0);
 
     // Normally we would use an AND mask, but this should be compatible with future MSN versions.
     if( clientCaps >= Contact::MSN_CAP_MSN60 )
